@@ -24,7 +24,7 @@ def Basis(*args, **kwargs): #pragma: no cover
     return FunctionSpace(*args, **kwargs)
 
 def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
-                  domain=None, scaled=None, padding_factor=1,
+                  domain=None, scaled=None, padding_factor=1, basis=None,
                   dealias_direct=False, coordinates=None, **kw):
     """Return function space
 
@@ -43,20 +43,41 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
         - ``Hermite`` or ``H``
         - ``Jacobi`` or ``J``
 
-    bc : str or tuple, optional
+    bc : tuple or dict, optional
         Choose one of
 
-        - 2-tuple (a, b) - Dirichlet boundary condition with
+        * 2-tuple of numbers (a, b) - Dirichlet boundary condition with
           :math:`v(-1)=a` and :math:`v(1)=b`.
-        - 4-tuple (a, b, 0, 0) - Biharmonic with the two non-zero Dirichlet
-          conditions :math:`v(-1)=a` and :math:`v(1)=b`.
-        - Dirichlet - Homogeneous Dirichlet.
-        - Neumann - Homogeneous Neumann.
-        - Biharmonic - Homogeneous Dirichlet and Neumann at both ends.
-        - UpperDirichlet - Homogeneous Dirichlet at x=1, nothing at x=-1.
-        - Polar - Homogeneous biharmonic, specifically used with polar coordinates.
-        - DirichletNeumann - Homogeneous Dirichlet at x=-1 and Neumann at x=1.
-        - NeumannDirichlet - Homogeneous Neumann at x=-1 and Dirichlet at x=1.
+
+        * (None, a) - Dirichlet on right boundary, nothing on left.
+
+        * 4-tuple (a, b, c, d) - Biharmonic with the two non-zero Dirichlet
+          conditions first :math:`v(-1)=a` and :math:`v(1)=b` and then
+          the two Neumann.
+
+        * dict with keys 'left' and 'right', for left and right boundaries,
+          and a list of 2-tuples to specify the condition. This is the most
+          general form, and all boundary conditions may be specified like this.
+          Specify Dirichlet on both ends with
+
+              {'left': [('D', a)], 'right': [('D', b)]}
+
+          Specify mixed Neumann and Dirichlet as
+
+              {'left': [('N', a)], 'right': [('D', b)]}
+
+          For both conditions on the right do
+
+              {'right': [('N', a), ('D', b)]}
+
+          Note that not all combinations are possible for biharmonic
+          problems. One combination that is possible is a fixed free beam
+          with
+
+              {'left': [('D', a), ('N', b)], 'right': [('N2', c), ('N3', d)]}
+
+          where 'N2' and 'N3' represent second and third derivatives.
+
     dtype : str or np.dtype, optional
         The datatype of physical space (input to forward transforms)
     quad : str, optional
@@ -84,6 +105,10 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
         The computational domain
     scaled : bool
         Whether to use scaled basis (only Legendre)
+    basis : str
+        Name of basis to use, if there are more than one possible basis for a given
+        boundary condition. For example, there are two Dirichlet bases for the
+        Chebyshev family: 'Heinricht' and 'ShenDirichlet'
     padding_factor : float, optional
         For padding backward transform (for dealiasing)
     dealias_direct : bool, optional
@@ -113,6 +138,59 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     par.update(kw)
     if domain is not None:
         par['domain'] = domain
+
+    def _process_bcs(bc, domain):
+
+        bcs = {'left': [], 'right': []}
+        df = 1
+        if domain is not None:
+            df = 2./(domain[1]-domain[0])
+        if isinstance(bc, tuple):
+            # Short form for Dirichlet or biharmonic with 2 or 4 numbers
+            assert len(bc) in (2, 4)
+            assert np.all([isinstance(i, (sp.Expr, Number)) or i is None for i in bc])
+            if len(bc) == 2:
+                if bc[0] is not None:
+                    bcs['left'] = [('D', bc[0])]
+                if bc[1] is not None:
+                    bcs['right'] = [('D', bc[1])]
+
+            if len(bc) == 4:
+                bcs['left'] = [('D', bc[0]), ('N', bc[2])]
+                bcs['right'] = [('D', bc[1]), ('N', bc[3])]
+            key = ['L'+bci[0] for bci in bcs['left']] + ['R'+bci[0] for bci in bcs['right']]
+
+        elif isinstance(bc, dict):
+            bcs = {k.lower(): list(v) if isinstance(v[0], (tuple, list)) else [v] for k, v in bc.items()}
+            bc = []
+            key = []
+            if 'left' in bcs:
+                bcs['left'].sort()
+                for bci in bcs['left']:
+                    if bci[0] == 'N':
+                        bc.append(bci[1]/df)
+                    elif bci[0] == 'N2':
+                        bc.append(bci[1]/df**2)
+                    elif bci[0] == 'N3':
+                        bc.append(bci[1]/df**3)
+                    else:
+                        bc.append(bci[1])
+                key += ['L'+bci[0] for bci in bcs['left']]
+            if 'right' in bcs:
+                bcs['right'].sort()
+                for bci in bcs['right']:
+                    if bci[0] == 'N':
+                        bc.append(bci[1]/df)
+                    elif bci[0] == 'N2':
+                        bc.append(bci[1]/df**2)
+                    elif bci[0] == 'N3':
+                        bc.append(bci[1]/df**3)
+                    else:
+                        bc.append(bci[1])
+                key += ['R'+bci[0] for bci in bcs['right']]
+
+        return key, tuple(bc)
+
     if family.lower() in ('fourier', 'f'):
         from shenfun import fourier
         if np.dtype(dtype).char in 'FDG':
@@ -125,45 +203,60 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
     elif family.lower() in ('chebyshev', 'c'):
         from shenfun import chebyshev
         if quad is not None:
-            assert quad in ('GC', 'GL')
+            assert quad in ('GC', 'GL', 'GU')
             par['quad'] = quad
 
-        if bc is None:
-            B = chebyshev.bases.Orthogonal
+        if scaled is not None:
+            assert isinstance(scaled, bool)
+            par['scaled'] = scaled
 
-        elif isinstance(bc, tuple):
-            assert len(bc) in (2, 4)
-            par['bc'] = bc
-            if len(bc) == 2:
-                B = chebyshev.bases.ShenDirichlet
-            else:
-                B = chebyshev.bases.ShenBiharmonic
+        # Boundary conditions abbreviated in dictionary keys as
+        #   left->L, right->R, Dirichlet->D, Neumann->N
+        # So LDRD means left Dirichlet, right Dirichlet
+        bases = {
+            '': chebyshev.bases.Orthogonal,
+            'LDRD': chebyshev.bases.ShenDirichlet,
+            'LNRN': chebyshev.bases.ShenNeumann,
+            'LDRN': chebyshev.bases.DirichletNeumann,
+            'LNRD': chebyshev.bases.NeumannDirichlet,
+            'RD': chebyshev.bases.UpperDirichlet,
+            'RDRN': chebyshev.bases.UpperDirichletNeumann,
+            'LDLNRDRN': chebyshev.bases.ShenBiharmonic
+        }
 
-        elif isinstance(bc, str):
-            if bc.lower() == 'dirichlet':
-                B = chebyshev.bases.ShenDirichlet
-            elif bc.lower() == 'neumann':
-                B = chebyshev.bases.ShenNeumann
-            elif bc.lower() == 'neumann2':
-                B = chebyshev.bases.SecondNeumann
-            elif bc.lower() == 'biharmonic':
-                B = chebyshev.bases.ShenBiharmonic
-            elif bc.lower() == 'upperdirichlet':
-                B = chebyshev.bases.UpperDirichlet
-            elif bc.lower() == 'bipolar':
-                B = chebyshev.bases.ShenBiPolar
-            elif bc.lower() == 'dirichletneumann':
-                B = chebyshev.bases.DirichletNeumann
-            elif bc.lower() == 'neumanndirichlet':
-                B = chebyshev.bases.NeumannDirichlet
+        if isinstance(bc, (tuple, dict)):
+            key, par['bc'] = _process_bcs(bc, domain)
+
+        elif bc is None:
+            key = ''
 
         else:
             raise NotImplementedError
 
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(chebyshev.bases, basis)
+        else:
+            B = bases[''.join(key)]
+
         return B(N, **par)
 
     elif family.lower() in ('legendre', 'l'):
         from shenfun import legendre
+
+        bases = {
+            '': legendre.bases.Orthogonal,
+            'LDRD': legendre.bases.ShenDirichlet,
+            'LNRN': legendre.bases.ShenNeumann,
+            'LDRN': legendre.bases.DirichletNeumann,
+            'LNRD': legendre.bases.NeumannDirichlet,
+            'RD': legendre.bases.UpperDirichlet,
+            'RDRN': legendre.bases.UpperDirichletNeumann,
+            'LNRDRN': legendre.bases.ShenBiPolar0,
+            'LDLNRDRN': legendre.bases.ShenBiharmonic,
+            'LDLNRN2RN3': legendre.bases.BeamFixedFree
+        }
+
         if quad is not None:
             assert quad in ('LG', 'GL')
             par['quad'] = quad
@@ -172,36 +265,20 @@ def FunctionSpace(N, family='Fourier', bc=None, dtype='d', quad=None,
             assert isinstance(scaled, bool)
             par['scaled'] = scaled
 
-        if bc is None:
-            B = legendre.bases.Orthogonal
+        if isinstance(bc, (tuple, dict)):
+            key, par['bc'] = _process_bcs(bc, domain)
 
-        elif isinstance(bc, tuple):
-            assert len(bc) in (2, 4)
-            par['bc'] = bc
-            if len(bc) == 2:
-                B = legendre.bases.ShenDirichlet
-            else:
-                B = legendre.bases.ShenBiharmonic
+        elif bc is None:
+            key = ''
 
-        elif isinstance(bc, str):
-            if bc.lower() == 'dirichlet':
-                B = legendre.bases.ShenDirichlet
-            elif bc.lower() == 'neumann':
-                B = legendre.bases.ShenNeumann
-            elif bc.lower() == 'biharmonic':
-                B = legendre.bases.ShenBiharmonic
-            elif bc.lower() == 'upperdirichlet':
-                B = legendre.bases.UpperDirichlet
-            elif bc.lower() == 'upperdirichletneumann':
-                B = legendre.bases.UpperDirichletNeumann
-            elif bc.lower() == 'bipolar':
-                B = legendre.bases.ShenBiPolar
-            elif bc.lower() == 'bipolar0':
-                B = legendre.bases.ShenBiPolar0
-            elif bc.lower() == 'dirichletneumann':
-                B = legendre.bases.DirichletNeumann
-            elif bc.lower() == 'neumanndirichlet':
-                B = legendre.bases.NeumannDirichlet
+        else:
+            raise NotImplementedError
+
+        if basis is not None:
+            assert isinstance(basis, str)
+            B = getattr(legendre.bases, basis)
+        else:
+            B = bases[''.join(key)]
 
         return B(N, **par)
 
@@ -469,7 +546,7 @@ class Expr:
                         for repl in replace:
                             assert len(repl) == 2
                             sc = sc.replace(*repl)
-                        sc = sp.simplify(sc)
+                        #sc = sp.simplify(sc)
                     scp = sp.latex(sc, symbol_names=symbol_names)
 
                     if isinstance(sc, sp.Add):
@@ -529,7 +606,7 @@ class Expr:
         0
 
         """
-        s = sp.S(0)
+
         ndim = self.dimensions
         if basis is None:
             basis = 'u'
@@ -553,31 +630,27 @@ class Expr:
                 basis = [basis]
             assert len(basis) == ndim**self.tensor_rank
 
-        assumptions = self.function_space().coors._assumptions
+        u = []
         for i, vec in enumerate(self.terms()):
+            s = sp.S(0)
             for j, term in enumerate(vec):
                 sc = self.scales()[i][j]
                 k = self.indices()[i][j]
                 b0 = basis[k]
                 tt = tuple([psi[n] for n, l in enumerate(term) for m in range(l)])
                 bi = 1
-                if self.expr_rank() > 0:
-                    bi = b[i%ndim]
                 if np.sum(term) > 0:
-                    sf = sc*bi*sp.diff(b0, *tt)
-                    #ss = sp.simplify(sp.refine(sf, assumptions))
-                    ss = sp.refine(sf, assumptions)
+                    ss = sc*bi*sp.diff(b0, *tt)
                     s += ss
 
                 else:
-                    sf = sc*b0*bi
-                    #ss = sp.simplify(sp.refine(, assumptions))
-                    ss = sp.refine(sf, assumptions)
+                    ss = sc*b0*bi
                     s += ss
+            u.append(s)
 
-        if isinstance(s, sp.ImmutableDenseNDimArray):
-            s = s.tolist()
-        return s
+        if len(u) == 1:
+            return u[0]
+        return u
 
     def eval(self, x, output_array=None):
         """Return expression evaluated on x
@@ -683,7 +756,6 @@ class Expr:
 
     def __mul__(self, a):
         sc = np.array(self.scales())
-        assumptions = self.function_space().coors._assumptions
         if self.expr_rank() == 0:
             sc = sc*sp.sympify(a)
 
@@ -698,7 +770,8 @@ class Expr:
 
         for i in range(sc.shape[0]):
             for j in range(sc.shape[1]):
-                sc[i, j] = sp.simplify(sp.refine(sc[i, j], assumptions))
+                sc[i, j] = self.function_space().coors.refine(sc[i, j])
+                sc[i, j] = sp.simplify(sc[i, j], measure=self.function_space().coors._measure)
 
         return Expr(self._basis, copy.deepcopy(self._terms), sc.tolist(), copy.deepcopy(self._indices))
 
@@ -707,7 +780,6 @@ class Expr:
 
     def __imul__(self, a):
         sc = np.array(self.scales())
-        assumptions = self.function_space().coors._assumptions
         if self.expr_rank() == 0:
             sc = sc*sp.sympify(a)
 
@@ -720,9 +792,11 @@ class Expr:
             else:
                 sc = sc*sp.sympify(a)
 
+        coors = self.function_space().coors
         for i in range(sc.shape[0]):
             for j in range(sc.shape[1]):
-                sc[i, j] = sp.simplify(sp.refine(sc[i, j], assumptions))
+                sc[i, j] = coors.refine(sc[i, j])
+                sc[i, j] = sp.simplify(sc[i, j], measure=coors._measure)
 
         self._scales = sc.tolist()
 
@@ -831,15 +905,15 @@ class Expr:
                     k = match[0]
                     assert inds[-1][k] == ind
                     scs[-1][k] += sc
-                    scs[-1][k] = sp.simplify(scs[-1][k])
-                    scs[-1][k] = sp.refine(scs[-1][k], self.function_space().coors._assumptions)
+                    scs[-1][k] = sp.simplify(scs[-1][k], measure=self.function_space().coors._measure)
+                    scs[-1][k] = self.function_space().coors.refine(scs[-1][k])
                     if scs[-1][k] == 0: # Remove if scale is zero
                         scs[-1].pop(k)
                         tms[-1].pop(k)
                         inds[-1].pop(k)
                     continue
-                sc = sp.simplify(sc)
-                sc = sp.refine(sc, self.function_space().coors._assumptions)
+                sc = sp.simplify(sc, measure=self.function_space().coors._measure)
+                sc = self.function_space().coors.refine(sc)
                 if not sc == 0:
                     tms[-1].append(term)
                     inds[-1].append(ind)
@@ -870,6 +944,19 @@ class Expr:
                     return False
         return True
 
+    def subs(self, a, b):
+        """Replace `a` with `b` in scales
+
+        Parameters
+        ----------
+        a : Sympy Symbol
+        b : Number
+        """
+        scs = self.scales()
+        for i, sci in enumerate(scs):
+            for j, scj in enumerate(sci):
+                scs[i][j] = scj.subs(a, b)
+
 
 class BasisFunction:
     """Base class for arguments to shenfun's Exprs
@@ -1111,7 +1198,7 @@ class ShenfunBaseArray(DistArray):
                 if isinstance(buf0, Number):
                     buffer.v[i] = buf0
                 elif hasattr(buf0, 'free_symbols'):
-                    sym0 = buf0.free_symbols
+                    sym0 = tuple(buf0.free_symbols)
                     m = []
                     for sym in sym0:
                         j = 'xyzrs'.index(str(sym))
@@ -1127,7 +1214,7 @@ class ShenfunBaseArray(DistArray):
 
         # if just one sympy expression
         if hasattr(buffer, 'free_symbols'):
-            sym0 = buffer.free_symbols
+            sym0 = tuple(buffer.free_symbols)
             mesh = space.local_mesh(True)
             m = []
             for sym in sym0:
