@@ -164,28 +164,31 @@ class _Fouriermatrix(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
     def solve(self, b, u=None, axis=0):
-        N = self.shape[0]
+        if self.measure == 1:
+            N = self.shape[0]
 
-        if u is None:
-            u = b
-        else:
-            assert u.shape == b.shape
+            if u is None:
+                u = b
+            else:
+                assert u.shape == b.shape
 
-        with np.errstate(divide='ignore'):
-            d = 1./self[0]
-        if isinstance(d, np.ndarray):
-            if np.isinf(d[0]):
-                d[0] = 0
-            if np.isinf(d[N//2]):
-                d[N//2] = 0
-            sl = [np.newaxis]*u.ndim
-            sl[axis] = slice(None)
-            u[:] = b*d[tuple(sl)]
-        else:
-            u[:] = b*d
+            with np.errstate(divide='ignore'):
+                d = 1./self[0]
+            if isinstance(d, np.ndarray):
+                if np.isinf(d[0]):
+                    d[0] = 0
+                if np.isinf(d[N//2]):
+                    d[N//2] = 0
+                sl = [np.newaxis]*u.ndim
+                sl[axis] = slice(None)
+                u[:] = b*d[tuple(sl)]
+            else:
+                u[:] = b*d
 
-        u /= self.scale
-        return u
+            u /= self.scale
+            return u
+
+        return SpectralMatrix.solve(self, b, u=u, axis=axis)
 
 
 class _FourierMatDict(dict):
@@ -208,11 +211,13 @@ class _FourierMatDict(dict):
 
 
 mat = _FourierMatDict({
-    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)**2): Acos2mat,
-    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)): Acosmat,
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)): Csinmat,
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(2*xp)/2): Csincosmat,
-    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)*sp.cos(xp)): Csincosmat,
-    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)**2): Bcos2mat,
-    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)): Bcosmat,
+    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)**2): functools.partial(Acos2mat, measure=sp.cos(xp)**2),
+    ((C2C, 0), (C2C, 2), (0, 2*np.pi), sp.cos(xp)): functools.partial(Acosmat, measure=sp.cos(xp)),
+    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)): functools.partial(Csinmat, measure=sp.sin(xp)),
+    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(2*xp)/2): functools.partial(Csincosmat, measure=sp.sin(2*xp)/2),
+    ((C2C, 0), (C2C, 1), (0, 2*np.pi), sp.sin(xp)*sp.cos(xp)): functools.partial(Csincosmat, measure=sp.sin(xp)*sp.cos(xp)),
+    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)**2): functools.partial(Bcos2mat, measure=sp.cos(xp)**2),
+    ((C2C, 0), (C2C, 0), (0, 2*np.pi), sp.cos(xp)): functools.partial(Bcosmat, measure=sp.cos(xp)),
 })
+
+#mat = _FourierMatDict({})
