@@ -17,7 +17,7 @@ The next two letters refer to the test and trialfunctions, respectively
     - Legendre:    `L`
     - Biharmonic:  `B`
 
-As such, there are 4 mass matrices, BDDmat, BNNmat, BLLmat and BBBmat,
+As such, there are 4 mass matrices, BSDSDmat, BSNSNmat, BLLmat and BSBSBmat,
 corresponding to the four bases above.
 
 A matrix may consist of different types of test and trialfunctions as long as
@@ -26,7 +26,7 @@ Neumann trial is named BDNmat.
 
 All matrices in this module may be looked up using the 'mat' dictionary,
 which takes test and trialfunctions along with the number of derivatives
-to be applied to each. As such the mass matrix BDDmat may be looked up
+to be applied to each. As such the mass matrix BSDSDmat may be looked up
 as
 
 >>> import numpy as np
@@ -55,10 +55,10 @@ generate the matrix as an inner product:
 >>> [np.all(abs(BM[k]-v) < 1e-7) for k, v in d.items()]
 [True, True, True]
 
-To see that this is in fact the BDDmat:
+To see that this is in fact the BSDSDmat:
 
 >>> print(BM.__class__)
-<class 'shenfun.legendre.matrices.BDDmat'>
+<class 'shenfun.legendre.matrices.BSDSDmat'>
 
 """
 from __future__ import division
@@ -75,13 +75,16 @@ from .la import TDMA
 from . import bases
 
 # Short names for instances of bases
-LB = bases.Orthogonal
+L  = bases.Orthogonal
 SD = bases.ShenDirichlet
 SB = bases.ShenBiharmonic
 SN = bases.ShenNeumann
-SU = bases.UpperDirichlet
-CD = bases.BCDirichlet
-CB = bases.BCBiharmonic
+UD = bases.UpperDirichlet
+DN = bases.DirichletNeumann
+BF = bases.BeamFixedFree
+
+BCD = bases.BCDirichlet
+BCB = bases.BCBiharmonic
 
 x = sp.symbols('x', real=True)
 xp = sp.symbols('x', real=True, positive=True)
@@ -106,10 +109,10 @@ class BLLmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
-        assert isinstance(trial[0], LB)
+        assert isinstance(test[0], L)
+        assert isinstance(trial[0], L)
         N = test[0].N
-        k = np.arange(N, dtype=np.float)
+        k = np.arange(N, dtype=float)
         d = {0: 2./(2.*k+1)}
         if test[0].quad == 'GL':
             d[0][-1] = 2./(N-1)
@@ -130,7 +133,7 @@ class BLLmat(SpectralMatrix):
         return u
 
 
-class BDDmat(SpectralMatrix):
+class BSDSDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -150,7 +153,7 @@ class BDDmat(SpectralMatrix):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
+        k = np.arange(N-2, dtype=float)
         d = {-2: -2./(2*k[2:] + 1),
               0: 2./(2.*k+1) + 2./(2*k+5)}
 
@@ -166,7 +169,7 @@ class BDDmat(SpectralMatrix):
         self.solve = TDMA(self)
 
 
-class BNNmat(SpectralMatrix):
+class BSNSNmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -186,7 +189,7 @@ class BNNmat(SpectralMatrix):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
+        k = np.arange(N-2, dtype=float)
         alpha = k*(k+1)/(k+2)/(k+3)
         d0 = 2./(2*k+1)
         d = {0: d0 + alpha**2*2./(2*(k+2)+1),
@@ -198,7 +201,7 @@ class BNNmat(SpectralMatrix):
         #self.solve = neumann_TDMA(self)
 
 
-class BBBmat(SpectralMatrix):
+class BSBSBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -209,7 +212,7 @@ class BBBmat(SpectralMatrix):
 
     .. math::
 
-        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+        j = 0, 1, ..., N-4 \text{ and } k = 0, 1, ..., N-4
 
     and :math:`\psi_k` is the Shen Legendre Biharmonic basis function.
 
@@ -219,7 +222,7 @@ class BBBmat(SpectralMatrix):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
-        k = np.arange(N, dtype=np.float)
+        k = np.arange(N, dtype=float)
         gk = (2*k+3)/(2*k+7)
         hk = -(1+gk)
         ek = 2./(2*k+1)
@@ -233,8 +236,48 @@ class BBBmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
         self.solve = PDMA(self)
 
+class BBFBFmat(SpectralMatrix):
+    r"""Mass matrix for inner product
 
-class BDLmat(SpectralMatrix):
+    .. math::
+
+        B_{kj} = (\psi_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-4 \text{ and } k = 0, 1, ..., N-4
+
+    and :math:`\psi_k` is the BeamFixedFree Biharmonic basis function.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], BF)
+        assert isinstance(trial[0], BF)
+        N = test[0].N
+        k = np.arange(N-4, dtype=float)
+        f1 = lambda k: 4*(2*k+3)/((k+3)**2)
+        f2 = lambda k: -(2*(k-1)*(k+1)*(k+6)*(2*k+5)/((k+3)**2*(k+4)*(2*k+7)))
+        f3 = lambda k: -4*(k+1)**2*(2*k+3)/((k+3)**2*(k+4)**2)
+        f4 = lambda k: (((k+1)/(k+3))*((k+2)/(k+4)))**2*(2*k+3)/(2*k+7)
+        d = {0: 2/(2*k+1)+f1(k)**2*2/(2*k+3)+f2(k)**2*2/(2*k+5)+f3(k)**2*2/(2*k+7)+f4(k)**2*2/(2*k+9),
+             1: (f1(k)*2/(2*k+3)+f1(k+1)*f2(k)*2/(2*k+5)+f2(k+1)*f3(k)*2/(2*k+7)+f3(k+1)*f4(k)*2/(2*k+9))[:-1],
+             2: (f2(k)*2/(2*k+5)+f1(k+2)*f3(k)*2/(2*k+7)+f2(k+2)*f4(k)*2/(2*k+9))[:-2],
+             3: (f3(k)*2/(2*k+7)+f1(k+3)*f4(k)*2/(2*k+9))[:-3],
+             4: (f4(k)*2/(2*k+9))[:-4]
+            }
+        d[-1] = d[1].copy()
+        d[-2] = d[2].copy()
+        d[-3] = d[3].copy()
+        d[-4] = d[4].copy()
+        if test[0].quad == 'GL':
+            k = N-5
+            d[0][-1] = 2/(2*k+1)+f1(k)**2*2/(2*k+3)+f2(k)**2*2/(2*k+5)+f3(k)**2*2/(2*k+7)+f4(k)**2*2/(N-1)
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+
+class BSDLmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -252,9 +295,9 @@ class BDLmat(SpectralMatrix):
     """
     def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
-        assert isinstance(trial[0], LB)
+        assert isinstance(trial[0], L)
         N = test[0].N
-        k = np.arange(N, dtype=np.float)
+        k = np.arange(N, dtype=float)
         sc = np.ones(N)
         if test[0].is_scaled():
             sc = 1. / np.sqrt(4*k+6)
@@ -265,7 +308,7 @@ class BDLmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class BLDmat(SpectralMatrix):
+class BLSDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -282,10 +325,10 @@ class BLDmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
+        assert isinstance(test[0], L)
         assert isinstance(trial[0], SD)
         N = test[0].N
-        k = np.arange(N, dtype=np.float)
+        k = np.arange(N, dtype=float)
         sc = np.ones(N)
         if trial[0].is_scaled():
             sc = 1. / np.sqrt(4*k+6)
@@ -296,8 +339,44 @@ class BLDmat(SpectralMatrix):
             d[-2][-1] = -2./(N-1)*sc[-3]
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
+class BDNDNmat(SpectralMatrix):
+    r"""Mass matrix for inner product
 
-class ADDmat(SpectralMatrix):
+    .. math::
+
+        B_{kj} = (\psi_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is a mixed Legendre Dirichlet/Neumann basis function.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], DN)
+        assert isinstance(trial[0], DN)
+        N = test[0].N
+        k = np.arange(N-2, dtype=float)
+        km = k[:-1]
+        kp = k[:-2]
+        d = {0: 2/(2*k+1) + 2*((2*k+3)/(k+2))/(k+2)**3 + 2*((k+1)/(k+2))**4/(2*k+5),
+            1: (2/(km+2)**2 - 2*((km+1)/(km+2))**2/(km+3)**2),
+            2: -2*((kp+1)/(kp+2))**2/(2*kp+5)
+        }
+        d[-1] = d[1].copy()
+        d[-2] = d[2].copy()
+
+        if test[0].quad == 'GL':
+            k = N-3
+            d[0][-1] = 2/(2*k+1) + 2*((2*k+3)/(k+2))/(k+2)**3 + 2*((k+1)/(k+2))**4/(N-1)
+
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+
+class ASDSDmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
     .. math::
@@ -317,7 +396,7 @@ class ADDmat(SpectralMatrix):
         assert isinstance(test[0], SD)
         assert isinstance(trial[0], SD)
         N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
+        k = np.arange(N-2, dtype=float)
         if not test[0].is_scaled():
             d = {0: 4*k+6}
         else:
@@ -366,7 +445,7 @@ class ADDmat(SpectralMatrix):
         return u
 
 
-class ANNmat(SpectralMatrix):
+class ASNSNmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
     .. math::
@@ -386,7 +465,7 @@ class ANNmat(SpectralMatrix):
         assert isinstance(test[0], SN)
         assert isinstance(trial[0], SN)
         N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
+        k = np.arange(N-2, dtype=float)
         alpha = k*(k+1)/(k+2)/(k+3)
         d0 = 2./(2*k+1)
         d = {0: d0*alpha*(k+0.5)*((k+2)*(k+3)-k*(k+1))}
@@ -402,7 +481,6 @@ class ANNmat(SpectralMatrix):
         else:
             assert u.shape == b.shape
 
-
         # Move axis to first
         if axis > 0:
             u = np.moveaxis(u, axis, 0)
@@ -415,16 +493,18 @@ class ANNmat(SpectralMatrix):
         sl = [np.newaxis]*bs.ndim
         sl[0] = slice(None)
         us[:] = bs*d[tuple(sl)]
+        u /= self.scale
+        self.testfunction[0].bc.set_boundary_dofs(u, True)
+        u[0] = self.testfunction[0].mean/(2/self.testfunction[0].domain_factor())
         if axis > 0:
             u = np.moveaxis(u, 0, axis)
             if u is not b:
                 b = np.moveaxis(b, axis, 0)
 
-        u /= self.scale
         return u
 
 
-class ABBmat(SpectralMatrix):
+class ASBSBmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
     .. math::
@@ -444,13 +524,128 @@ class ABBmat(SpectralMatrix):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
-        k = np.arange(N-4, dtype=np.float)
+        k = np.arange(N-4, dtype=float)
         gk = (2*k+3)/(2*k+7)
         d = {0: 2*(2*k+3)*(1+gk),
              2: -2*(2*k[:-2]+3)}
         d[-2] = d[2]
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+class ADNDNmat(SpectralMatrix):
+    r"""Stiffness matrix for inner product
+
+    .. math::
+
+        A_{kj} = (\psi'_j, \psi'_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the mixed Legendre Dirichlet/Neumann basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], DN)
+        assert isinstance(trial[0], DN)
+        N = test[0].N
+        k = np.arange(N-2, dtype=float)
+        d = {0: ((k+1)/(k+2))**2*((k+2)*(k+3)- k*(k+1))}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+    def solve(self, b, u=None, axis=0):
+        N = self.shape[0] + 2
+        assert N == b.shape[axis]
+        s = self.trialfunction[0].slice()
+
+        if u is None:
+            u = b
+        else:
+            assert u.shape == b.shape
+
+
+        # Move axis to first
+        if axis > 0:
+            u = np.moveaxis(u, axis, 0)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+
+        bs = b[s]
+        us = u[s]
+        d = 1./self[0]
+        sl = [np.newaxis]*bs.ndim
+        sl[0] = slice(None)
+        us[:] = bs*d[tuple(sl)]
+        u /= self.scale
+        self.testfunction[0].bc.set_boundary_dofs(u, True)
+
+        if axis > 0:
+            u = np.moveaxis(u, 0, axis)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+
+        return u
+
+class SBFBFmat(SpectralMatrix):
+    r"""Biharmonic matrix for inner product
+
+    .. math::
+
+        S_{kj} = (\psi''_j, \psi''_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-4 \text{ and } k = 0, 1, ..., N-4
+
+    and :math:`\psi_k` is the BeamFixedFree basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], BF)
+        assert isinstance(trial[0], BF)
+        N = test[0].N
+        k = np.arange(N-4, dtype=float)
+        f4 = (((k+1)/(k+3))*((k+2)/(k+4)))**2*(2*k+3)/(2*k+7)
+        d = {0: f4*(k+2.5)*((k+4)*(k+5)-(k+2)*(k+3))*((k+2)*(k+3)-k*(k+1))}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+    def solve(self, b, u=None, axis=0):
+        N = self.shape[0] + 4
+        assert N == b.shape[axis]
+        s = self.trialfunction[0].slice()
+
+        if u is None:
+            u = b
+        else:
+            assert u.shape == b.shape
+
+        # Move axis to first
+        if axis > 0:
+            u = np.moveaxis(u, axis, 0)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+
+        bs = b[s]
+        us = u[s]
+        d = 1./self[0]
+        sl = [np.newaxis]*bs.ndim
+        sl[0] = slice(None)
+        us[:] = bs*d[tuple(sl)]
+        u /= self.scale
+        self.testfunction[0].bc.set_boundary_dofs(u, True)
+
+        if axis > 0:
+            u = np.moveaxis(u, 0, axis)
+            if u is not b:
+                b = np.moveaxis(b, axis, 0)
+
+        return u
+
+
 class GLLmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
@@ -468,10 +663,10 @@ class GLLmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
-        assert isinstance(trial[0], LB)
+        assert isinstance(test[0], L)
+        assert isinstance(trial[0], L)
         N = test[0].N
-        k = np.arange(N, dtype=np.float)
+        k = np.arange(N, dtype=float)
         d = {}
         for j in range(2, N, 2):
             jj = j if trial[1] else -j
@@ -495,7 +690,7 @@ class GLLmat(SpectralMatrix):
             c = super(GLLmat, self).matvec(v, c, format=format, axis=axis)
         return c
 
-class SBBmat(SpectralMatrix):
+class SSBSBmat(SpectralMatrix):
     r"""Stiffness matrix for inner product
 
     .. math::
@@ -515,7 +710,7 @@ class SBBmat(SpectralMatrix):
         assert isinstance(test[0], SB)
         assert isinstance(trial[0], SB)
         N = test[0].N
-        k = np.arange(N-4, dtype=np.float)
+        k = np.arange(N-4, dtype=float)
         d = {0: 2*(2*k+3)**2*(2*k+5)}
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
@@ -537,8 +732,8 @@ class CLLmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
-        assert isinstance(trial[0], LB)
+        assert isinstance(test[0], L)
+        assert isinstance(trial[0], L)
         N = test[0].N
         d = {}
         for i in range(1, N, 2):
@@ -591,8 +786,8 @@ class CLLmatT(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
-        assert isinstance(trial[0], LB)
+        assert isinstance(test[0], L)
+        assert isinstance(trial[0], L)
         N = test[0].N
         d = {}
         for i in range(-1, -N, -2):
@@ -600,7 +795,7 @@ class CLLmatT(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class CLDmat(SpectralMatrix):
+class CLSDmat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -617,17 +812,17 @@ class CLDmat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, measure=1):
-        assert isinstance(test[0], LB)
+        assert isinstance(test[0], L)
         assert isinstance(trial[0], SD)
         N = test[0].N
         d = {-1: -2}
         if trial[0].is_scaled():
-            k = np.arange(N-2, dtype=np.float)
+            k = np.arange(N-2, dtype=float)
             d[-1] = -2. / np.sqrt(4*k+6)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class CDLmat(SpectralMatrix):
+class CSDLmat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -645,16 +840,16 @@ class CDLmat(SpectralMatrix):
     """
     def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
-        assert isinstance(trial[0], LB)
+        assert isinstance(trial[0], L)
         N = test[0].N
         d = {1: -2}
         if test[0].is_scaled():
-            k = np.arange(N-2, dtype=np.float)
+            k = np.arange(N-2, dtype=float)
             d[1] = -2. / np.sqrt(4*k+6)
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class CDDmat(SpectralMatrix):
+class CSDSDmat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -676,13 +871,13 @@ class CDDmat(SpectralMatrix):
         N = test[0].N
         d = {-1: -2, 1: 2}
         if trial[0].is_scaled():
-            k = np.arange(N-2, dtype=np.float)
+            k = np.arange(N-2, dtype=float)
             d[-1] = -2. / np.sqrt(4*k[:-1]+6)
             d[1] = 2. / np.sqrt(4*k[:-1]+6)
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class ADDrp1mat(SpectralMatrix):
+class ASDSDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -708,7 +903,7 @@ class ADDrp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class ADD2rp1mat(SpectralMatrix):
+class ASDSD2rp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -734,7 +929,7 @@ class ADD2rp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class ADD2Trp1mat(SpectralMatrix):
+class ASDSD2Trp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -760,7 +955,7 @@ class ADD2Trp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class AUUrp1mat(SpectralMatrix):
+class AUDUDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -777,15 +972,173 @@ class AUUrp1mat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], SU)
-        assert isinstance(trial[0], SU)
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
         assert test[0].quad == 'LG'
         k = np.arange(test[0].N-1)
         d = {0: 2*k+2}
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
+class AUDUDrp1smat(SpectralMatrix):
+    r"""Matrix for inner product
 
-class BUUrp1mat(SpectralMatrix):
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j'(x) \psi_k'(x) (1+x)**2 dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        #d = {0: 4*k**2*(k+1)/(2*k+1)+4*(k+1)**2*(k+2)/(2*k+3)-4*k*(k+1),
+        #     1: 2*(k[:-1]+1)*(k[:-1]+2)-4*(k[:-1]+1)**2*(k[:-1]+2)/(2*k[:-1]+3)}
+        d = {0: 2*(k+1)**2*(1/(2*k+1)+1/(2*k+3)),
+             1: 2*k[1:]*(k[1:]+1)/(2*k[1:]+1)}
+        d[-1] = d[1].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class GUDUDrp1smat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k''(x) (1+x)**2 dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        d = {0: -2*(k+1)*((k-1)/(2*k+1) + (k+3)/(2*k+3)),
+             1: -2*(k[1:]+1)*(k[1:]+2)/(2*k[1:]+1),
+             -1: -2*k[:-1]*(k[:-1]+1)/(2*k[:-1]+3)}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class BUDUDrp1smat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        B_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k(x) (1+x)**2 dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        #a00 = 2/(2*k+1)
+        #a11 = 2/(2*k+3)
+        #a22 = 2/(2*k+5)
+        #c00 = ((k+1)**2/(2*k+1)/(2*k+3) + k**2/(2*k+1)/(2*k-1))*a00
+        #c11 = ((k+2)**2/(2*k+3)/(2*k+5) + (k+1)**2/(2*k+3)/(2*k+1))*a11
+        #c02 = (k+2)*(k+1)/(2*k+5)/(2*k+3)*a00
+        #c13 = ((k+3)*(k+2)/(2*k+7)/(2*k+5))*a11
+        #b01 = (k+1)/(2*k+3)*a00
+        #b12 = (k+2)/(2*k+5)*a11
+
+        #d = {0: a00+c00-4*b01+a11+c11,
+        #     1: (2*b01-c02-a11-c11+2*b12)[:-1],
+        #     -1: (2*b01-c02-a11-c11+2*b12)[:-1],
+        #     2: (c02-2*b12+c13)[:-2],
+        #     -2: (c02-2*b12+c13)[:-2],
+        #     3: -c13[:-3].copy(),
+        #     -3: -c13[:-3].copy()}
+
+        d = {0: (k/(2*k+1))**2*(2/(2*k-1) + 2/(2*k+3)) + ((k+2)/(2*k+3))**2 * (2/(2*k+1)+2/(2*k+5)),
+             1: 2*k[1:]*(k[1:]+1)/(2*k[1:]+1)**2*(1/(2*k[1:]-1)+1/(2*k[1:]+3)) - 2*(k[1:]+2)*(k[1:]-1)/(2*k[1:]+3)/(2*k[1:]+1)/(2*k[1:]-1),
+             2: -2*k[2:]*(k[2:]-2)/(2*k[2:]+1)/(2*k[2:]-1)/(2*k[2:]-3)-2*k[2:]*(k[2:]+2)/(2*k[2:]+3)/(2*k[2:]+1)/(2*k[2:]-1),
+             3: -2*k[3:]*(k[3:]-1)/(2*k[3:]+1)/(2*k[3:]-1)/(2*k[3:]-3)}
+        d[-1] = d[1].copy()
+        d[-2] = d[2].copy()
+        d[-3] = d[3].copy()
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+class CUDUDrp1mat(SpectralMatrix):
+    r"""Matrix for inner product
+
+    .. math::
+
+        A_{kj} = \int_{-1}^{1} \psi_j(x) \psi_k'(x) (1+x) dx
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-2 \text{ and } k = 0, 1, ..., N-2
+
+    and :math:`\psi_k` is the Upper Dirichlet basis function.
+
+    """
+    def __init__(self, test, trial, scale=1, measure=1):
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
+        assert test[0].quad == 'LG'
+        k = np.arange(test[0].N-1)
+        d = {0: -2*(k+1)/(2*k+1)+2*(k+1)/(2*k+3),
+             1:  2*(k[1:]+1)/(2*k[1:]+1),
+             -1: -2*(k[:-1]+1)/(2*k[:-1]+3)}
+        SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
+
+
+class BUDUDmat(SpectralMatrix):
+    r"""Mass matrix for inner product
+
+    .. math::
+
+        B_{kj} = (\psi_j, \psi_k)_w
+
+    where
+
+    .. math::
+
+        j = 0, 1, ..., N-1 \text{ and } k = 0, 1, ..., N-1
+
+    and :math:`\psi_k` is the Legendre UpperDirichlet basis function.
+
+    """
+    def __init__(self, test, trial, measure=1):
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
+        N = test[0].N
+        k = np.arange(N-1, dtype=float)
+        d = {-1: -2./(2*k[1:] + 1),
+              0: 2./(2.*k+1) + 2./(2*k+3)}
+
+        if test[0].quad == 'GL':
+            d[0][-1] = 2./(2*(N-2)+1) + 2./(N-1)
+
+        d[1] = d[-1]
+        SpectralMatrix.__init__(self, d, test, trial, measure=measure)
+
+class BUDUDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -802,8 +1155,8 @@ class BUUrp1mat(SpectralMatrix):
 
     """
     def __init__(self, test, trial, scale=1, measure=1):
-        assert isinstance(test[0], SU)
-        assert isinstance(trial[0], SU)
+        assert isinstance(test[0], UD)
+        assert isinstance(trial[0], UD)
         assert test[0].quad == 'LG'
         k = np.arange(test[0].N-1)
         d = {0: 2*k+2}
@@ -815,7 +1168,7 @@ class BUUrp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class BDD1orp1mat(SpectralMatrix):
+class BSDSD1orp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -841,7 +1194,7 @@ class BDD1orp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class BDDrp1mat(SpectralMatrix):
+class BSDSDrp1mat(SpectralMatrix):
     r"""Matrix for inner product
 
     .. math::
@@ -873,7 +1226,7 @@ class BDDrp1mat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, scale=scale, measure=measure)
 
 
-class BCDmat(SpectralMatrix):
+class BSDBCDmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -892,9 +1245,9 @@ class BCDmat(SpectralMatrix):
     """
     def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SD)
-        assert isinstance(trial[0], CD)
+        assert isinstance(trial[0], BCD)
         N = test[0].N
-        k = np.arange(N-2, dtype=np.float)
+        k = np.arange(N-2, dtype=float)
         if not test[0].is_scaled():
             d = {0: np.array([1, 1./3.]),
                  1: np.array([1.0]),
@@ -907,7 +1260,7 @@ class BCDmat(SpectralMatrix):
         SpectralMatrix.__init__(self, d, test, trial, measure=measure)
 
 
-class BCBmat(SpectralMatrix):
+class BSBBCBmat(SpectralMatrix):
     r"""Mass matrix for inner product
 
     .. math::
@@ -926,9 +1279,9 @@ class BCBmat(SpectralMatrix):
     """
     def __init__(self, test, trial, measure=1):
         assert isinstance(test[0], SB)
-        assert isinstance(trial[0], CB)
+        assert isinstance(trial[0], BCB)
         N = test[0].N
-        k = np.arange(N-4, dtype=np.float)
+        k = np.arange(N-4, dtype=float)
         d = {0: np.array([1, 4/9, -1/15, 1/35]),
              1: np.array([1, -1/9, 1/15]),
              2: np.array([3/7, -1/9]),
@@ -964,46 +1317,65 @@ class _LegMatDict(dict):
         return matrix
 
 mat = _LegMatDict({
-    ((LB, 0), (LB, 0)): BLLmat,
-    ((LB, 0), (LB, 1)): CLLmat,
-    ((LB, 1), (LB, 0)): CLLmatT,
-    ((LB, 0), (SD, 1)): CLDmat,
-    ((SD, 1), (LB, 0)): CDLmat,
-    ((SD, 0), (SD, 1)): CDDmat,
-    ((SD, 1), (SD, 0)): functools.partial(CDDmat, scale=-1.),
-    ((SD, 0), (SD, 0)): BDDmat,
-    ((SB, 0), (SB, 0)): BBBmat,
-    ((SN, 0), (SN, 0)): BNNmat,
-    ((SD, 0), (LB, 0)): BDLmat,
-    ((LB, 0), (SD, 0)): BLDmat,
-    ((SD, 1), (SD, 1)): ADDmat,
-    ((SD, 2), (SD, 0)): functools.partial(ADDmat, scale=-1.),
-    ((SD, 0), (SD, 2)): functools.partial(ADDmat, scale=-1.),
-    ((SN, 1), (SN, 1)): ANNmat,
-    ((SN, 2), (SN, 0)): functools.partial(ANNmat, scale=-1.),
-    ((SN, 0), (SN, 2)): functools.partial(ANNmat, scale=-1.),
-    ((LB, 2), (LB, 0)): GLLmat,
-    ((LB, 0), (LB, 2)): GLLmat,
-    ((SB, 2), (SB, 2)): SBBmat,
-    ((SB, 1), (SB, 1)): ABBmat,
-    ((SB, 0), (SB, 2)): functools.partial(ABBmat, scale=-1.),
-    ((SB, 2), (SB, 0)): functools.partial(ABBmat, scale=-1.),
-    ((SB, 0), (SB, 4)): SBBmat,
-    ((SB, 4), (SB, 0)): SBBmat,
-    ((SD, 1), (SD, 1), (-1, 1), 1+x): ADDrp1mat,
-    ((SD, 0), (SD, 2), (-1, 1), 1+x): ADD2rp1mat,
-    ((SD, 2), (SD, 0), (-1, 1), 1+x): ADD2Trp1mat,
-    ((SD, 0), (SD, 2), (0, 1), xp): functools.partial(ADD2rp1mat, scale=0.5),
-    ((SD, 2), (SD, 0), (0, 1), xp): functools.partial(ADD2Trp1mat, scale=0.5),
-    ((SD, 1), (SD, 1), (0, 1), xp): functools.partial(ADDrp1mat, scale=0.5),
-    ((SD, 0), (SD, 0), (-1, 1), 1+x): BDDrp1mat,
-    ((SD, 0), (SD, 0), (0, 1), xp): functools.partial(BDDrp1mat, scale=0.5),
-    ((SD, 0), (SD, 0), (-1, 1), 1/(1+x)): BDD1orp1mat,
-    ((SD, 0), (SD, 0), (0, 1), 1/xp): functools.partial(BDD1orp1mat, scale=2),
-    ((SU, 1), (SU, 1), (-1, 1), 1+x): AUUrp1mat,
-    ((SU, 1), (SU, 1), (0, 1), xp): functools.partial(AUUrp1mat, scale=0.5),
-    ((SU, 0), (SU, 0), (-1, 1), 1+x): BUUrp1mat,
-    ((SU, 0), (SU, 0), (0, 1), xp): functools.partial(BUUrp1mat, scale=0.5),
-    ((SD, 0), (CD, 0)): BCDmat,
-    ((SB, 0), (CB, 0)): BCBmat,
+    ((L,  0), (L,  0)): BLLmat,
+    ((L,  0), (L,  1)): CLLmat,
+    ((L,  1), (L,  0)): CLLmatT,
+    ((L,  0), (SD, 1)): CLSDmat,
+    ((SD, 1), (L,  0)): CSDLmat,
+    ((SD, 0), (SD, 1)): CSDSDmat,
+    ((SD, 1), (SD, 0)): functools.partial(CSDSDmat, scale=-1.),
+    ((SD, 0), (SD, 0)): BSDSDmat,
+    ((SB, 0), (SB, 0)): BSBSBmat,
+    ((SN, 0), (SN, 0)): BSNSNmat,
+    ((SD, 0), (L,  0)): BSDLmat,
+    ((L,  0), (SD, 0)): BLSDmat,
+    ((SD, 1), (SD, 1)): ASDSDmat,
+    ((SD, 2), (SD, 0)): functools.partial(ASDSDmat, scale=-1.),
+    ((SD, 0), (SD, 2)): functools.partial(ASDSDmat, scale=-1.),
+    ((SN, 1), (SN, 1)): ASNSNmat,
+    ((SN, 2), (SN, 0)): functools.partial(ASNSNmat, scale=-1.),
+    ((SN, 0), (SN, 2)): functools.partial(ASNSNmat, scale=-1.),
+    ((L,  2), (L,  0)): GLLmat,
+    ((L,  0), (L,  2)): GLLmat,
+    ((SB, 2), (SB, 2)): SSBSBmat,
+    ((SB, 1), (SB, 1)): ASBSBmat,
+    ((SB, 0), (SB, 2)): functools.partial(ASBSBmat, scale=-1.),
+    ((SB, 2), (SB, 0)): functools.partial(ASBSBmat, scale=-1.),
+    ((SB, 0), (SB, 4)): SSBSBmat,
+    ((SB, 4), (SB, 0)): SSBSBmat,
+    ((SD, 1), (SD, 1), (-1, 1), 1+x): functools.partial(ASDSDrp1mat, measure=1+x),
+    ((SD, 0), (SD, 2), (-1, 1), 1+x): functools.partial(ASDSD2rp1mat, measure=1+x),
+    ((SD, 2), (SD, 0), (-1, 1), 1+x): functools.partial(ASDSD2Trp1mat, measure=1+x),
+    ((SD, 0), (SD, 2), (0, 1), xp): functools.partial(ASDSD2rp1mat, scale=0.5, measure=xp),
+    ((SD, 2), (SD, 0), (0, 1), xp): functools.partial(ASDSD2Trp1mat, scale=0.5, measure=xp),
+    ((SD, 1), (SD, 1), (0, 1), xp): functools.partial(ASDSDrp1mat, scale=0.5, measure=xp),
+    ((SD, 0), (SD, 0), (-1, 1), 1+x): functools.partial(BSDSDrp1mat, measure=1+x),
+    ((SD, 0), (SD, 0), (0, 1), xp): functools.partial(BSDSDrp1mat, scale=0.5, measure=xp),
+    ((SD, 0), (SD, 0), (-1, 1), 1/(1+x)): functools.partial(BSDSD1orp1mat, measure=1/(1+x)),
+    ((SD, 0), (SD, 0), (0, 1), 1/xp): functools.partial(BSDSD1orp1mat, scale=2, measure=1/xp),
+    ((UD, 1), (UD, 1), (-1, 1), 1+x): functools.partial(AUDUDrp1mat, measure=(1+x)),
+    ((UD, 1), (UD, 1), (0, 1), xp): functools.partial(AUDUDrp1mat, scale=0.5, measure=xp),
+    ((UD, 0), (UD, 0), (-1, 1), 1+x): functools.partial(BUDUDrp1mat, measure=(1+x)),
+    ((UD, 0), (UD, 0), (0, 1), xp): functools.partial(BUDUDrp1mat, scale=0.5, measure=xp),
+    ((UD, 1), (UD, 1), (-1, 1), (1+x)**2): functools.partial(AUDUDrp1smat, measure=(1+x)**2),
+    ((UD, 1), (UD, 1), (0, 1), xp**2): functools.partial(AUDUDrp1smat, scale=0.25, measure=xp**2),
+    ((UD, 0), (UD, 2), (-1, 1), (1+x)**2): functools.partial(GUDUDrp1smat, measure=(1+x)**2),
+    ((UD, 0), (UD, 2), (0, 1), xp**2): functools.partial(GUDUDrp1smat, scale=0.25, measure=xp**2),
+    ((UD, 0), (UD, 1), (-1, 1), (1+x)): functools.partial(CUDUDrp1mat, measure=(1+x)),
+    ((UD, 0), (UD, 1), (0, 1), xp): functools.partial(CUDUDrp1mat, scale=0.5, measure=xp),
+    ((UD, 0), (UD, 0), (-1, 1), (1+x)**2): functools.partial(BUDUDrp1smat, measure=(1+x)**2),
+    ((UD, 0), (UD, 0), (0, 1), xp**2): functools.partial(BUDUDrp1smat, scale=0.25, measure=xp**2),
+    ((UD, 0), (UD, 0)): BUDUDmat,
+    ((SD, 0), (BCD, 0)): BSDBCDmat,
+    ((SB, 0), (BCB, 0)): BSBBCBmat,
+    ((DN, 0), (DN, 0)): BDNDNmat,
+    ((DN, 1), (DN, 1)): ADNDNmat,
+    ((DN, 2), (DN, 0)): functools.partial(ADNDNmat, scale=-1.),
+    ((DN, 0), (DN, 2)): functools.partial(ADNDNmat, scale=-1.),
+    ((BF, 4), (BF, 0)): SBFBFmat,
+    ((BF, 0), (BF, 4)): SBFBFmat,
+    ((BF, 2), (BF, 2)): SBFBFmat,
+    ((BF, 0), (BF, 0)): BBFBFmat
     })
+
+#mat = _LegMatDict({})
