@@ -4,6 +4,7 @@ This module contains classes for working with sparse matrices.
 """
 from __future__ import division
 from copy import deepcopy
+from collections.abc import Mapping, MutableMapping
 from numbers import Number, Integral
 import numpy as np
 import sympy as sp
@@ -13,12 +14,13 @@ from mpi4py import MPI
 from .utilities import integrate_sympy
 
 __all__ = ['SparseMatrix', 'SpectralMatrix', 'extract_diagonal_matrix',
-           'check_sanity', 'get_dense_matrix', 'TPMatrix', 'BlockMatrix',
-           'Identity', 'get_dense_matrix_sympy']
+           'extract_bc_matrices', 'check_sanity', 'get_dense_matrix',
+           'TPMatrix', 'BlockMatrix', 'BlockMatrices', 'Identity',
+           'get_dense_matrix_sympy', 'get_dense_matrix_quadpy']
 
 comm = MPI.COMM_WORLD
 
-class SparseMatrix(dict):
+class SparseMatrix(MutableMapping):
     r"""Base class for sparse matrices.
 
     The data is stored as a dictionary, where keys and values are, respectively,
@@ -34,6 +36,12 @@ class SparseMatrix(dict):
     scale : number, optional
         Scale matrix with this number
 
+    Note
+    ----
+    The dictionary can use a function to generate its values. See, e.g.,
+    the ASDSDmat matrix, where diagonals for keys > 2 are functions that return
+    the proper diagonal when looked up.
+
     Examples
     --------
     A tridiagonal matrix of shape N x N could be created as
@@ -53,11 +61,13 @@ class SparseMatrix(dict):
     ...       1: np.ones(N-1)}
     >>> SparseMatrix(d, (N, N))
     {-1: array([1., 1., 1.]), 0: array([-2., -2., -2., -2.]), 1: array([1., 1., 1.])}
+
     """
     # pylint: disable=redefined-builtin, missing-docstring
 
     def __init__(self, d, shape, scale=1.0):
-        dict.__init__(self, d)
+        self._storage = dict(d)
+        #dict.__init__(self, d)
         self.shape = shape
         self._diags = dia_matrix((1, 1))
         self.scale = scale
@@ -82,6 +92,7 @@ class SparseMatrix(dict):
              - python - Use numpy and vectorization
              - self - To be implemented in subclass
              - cython - Cython implementation that may be implemented in subclass
+             - numba - Numba implementation that may be implemented in subclass
         axis : int, optional
             The axis over which to take the matrix vector product
 
@@ -146,6 +157,24 @@ class SparseMatrix(dict):
 
         return self._diags
 
+    def __getitem__(self, key):
+        v = self._storage[key]
+        if hasattr(v, '__call__'):
+            return v(key)
+        return v
+
+    def __delitem__(self, key):
+        del self._storage[key]
+
+    def __setitem__(self, key, val):
+        self._storage[key] = val
+
+    def __iter__(self):
+        return iter(self._storage)
+
+    def __len__(self):
+        return len(self._storage)
+
     def __eq__(self, a):
         if self.shape != a.shape:
             return False
@@ -221,7 +250,7 @@ class SparseMatrix(dict):
 
             else:
                 f = SparseMatrix(deepcopy(dict(self)), self.shape, self.scale)
-                assert isinstance(d, dict)
+                assert isinstance(d, Mapping)
                 for key, val in d.items():
                     if key in f:
                         # Check if symmetric and make copy if necessary
@@ -236,7 +265,7 @@ class SparseMatrix(dict):
 
     def __iadd__(self, d):
         """self.__iadd__(d) <==> self += d"""
-        assert isinstance(d, dict)
+        assert isinstance(d, Mapping)
         assert d.shape == self.shape
         #if self == d:
         #    self.scale += d.scale
@@ -264,10 +293,14 @@ class SparseMatrix(dict):
 
     def __sub__(self, d):
         """Return copy of self.__sub__(d) <==> self-d"""
-        assert isinstance(d, dict)
+        assert isinstance(d, Mapping)
         if self == d:
             f = SparseMatrix(deepcopy(dict(self)), self.shape,
                              self.scale-d.scale)
+
+        elif abs(self.scale) < 1e-16:
+            f = SparseMatrix(deepcopy(dict(d)), d.shape, -d.scale)
+
         else:
             f = SparseMatrix(deepcopy(dict(self)), self.shape, self.scale)
             for key, val in d.items():
@@ -284,10 +317,17 @@ class SparseMatrix(dict):
 
     def __isub__(self, d):
         """self.__isub__(d) <==> self -= d"""
-        assert isinstance(d, dict)
+        assert isinstance(d, Mapping)
         assert d.shape == self.shape
         if self == d:
             self.scale -= d.scale
+
+        elif abs(self.scale) < 1e-16:
+            self.clear()
+            for key, val in d.items():
+                self[key] = val
+            self.scale = -d.scale
+
         else:
             for key, val in d.items():
                 if key in self:
@@ -360,7 +400,8 @@ class SparseMatrix(dict):
 
         if b.ndim == 1:
             if use_lu:
-                if b.dtype.char in 'FDG':
+
+                if b.dtype.char in 'FDG' and self._lu.U.dtype.char in 'fdg':
                     u.real[:] = self._lu.solve(b.real)
                     u.imag[:] = self._lu.solve(b.imag)
                 else:
@@ -371,7 +412,7 @@ class SparseMatrix(dict):
             N = b.shape[0]
             P = np.prod(b.shape[1:])
             if use_lu:
-                if b.dtype.char in 'FDG':
+                if b.dtype.char in 'FDG' and self._lu.U.dtype.char in 'fdg':
                     u.real[:] = self._lu.solve(b.real.reshape((N, P))).reshape(u.shape)
                     u.imag[:] = self._lu.solve(b.imag.reshape((N, P))).reshape(u.shape)
                 else:
@@ -401,6 +442,11 @@ class SparseMatrix(dict):
             return True
         return False
 
+    @property
+    def issymmetric(self):
+        M = self.diags()
+        return (abs(M-M.T) > 1e-8).nnz == 0
+
     def clean_diagonals(self, reltol=1e-8):
         """Eliminate essentially zerovalued diagonals
 
@@ -494,11 +540,15 @@ class SpectralMatrix(SparseMatrix):
         shape = (test[0].dim(), trial[0].dim())
         if d == {}:
             D = get_dense_matrix(test, trial, measure)[:shape[0], :shape[1]]
+            #D = get_denser_matrix(test, trial, measure)[:shape[0], :shape[1]]
             #D = get_dense_matrix_sympy(test, trial, measure)[:shape[0], :shape[1]]
             d = extract_diagonal_matrix(D)
         SparseMatrix.__init__(self, d, shape, scale)
         if shape[0] == shape[1]:
-            from shenfun.la import Solve
+            from shenfun.la import Solve, NeumannSolve
+            #if test[0].use_fixed_gauge:
+            #    self.solver = NeumannSolve(self, test[0])
+            #else:
             self.solver = Solve(self, test[0])
 
     def matvec(self, v, c, format='csr', axis=0):
@@ -506,7 +556,7 @@ class SpectralMatrix(SparseMatrix):
         ss = [slice(None)]*len(v.shape)
         ss[axis] = u.slice()
         c = super(SpectralMatrix, self).matvec(v[tuple(ss)], c, format=format, axis=axis)
-        if self.testfunction[0].__class__.__name__ == 'ShenNeumann':
+        if self.testfunction[0].use_fixed_gauge:
             ss[axis] = 0
             c[tuple(ss)] = 0
         return c
@@ -567,11 +617,15 @@ class SpectralMatrix(SparseMatrix):
             return False
         if self.get_key() != a.get_key():
             return False
-        if not np.array(list(self.keys())).shape == np.array(list(a.keys())).shape:
+        sl = np.array(list(self.keys()))
+        sa = np.array(list(a.keys()))
+        if not sl.shape == sa.shape:
             return False
-        #if not np.all(np.array(list(self.keys())) == np.array(list(a.keys()))):
-        #    return False
-        if np.linalg.norm(self[list(self.keys())[0]] - a[list(self.keys())[0]]) > 1e-8:
+        sl.sort()
+        sa.sort()
+        if not np.linalg.norm((sl-sa)**2) == 0:
+            return False
+        if np.linalg.norm(sl[0] - sa[0]) > 1e-8:
             return False
         return True
 
@@ -597,7 +651,7 @@ class SpectralMatrix(SparseMatrix):
 
     def __add__(self, y):
         """Return copy of self.__add__(y) <==> self+y"""
-        assert isinstance(y, dict)
+        assert isinstance(y, Mapping)
         if self == y:
             f = SpectralMatrix(deepcopy(dict(self)), self.testfunction,
                                self.trialfunction, self.scale+y.scale)
@@ -615,7 +669,7 @@ class SpectralMatrix(SparseMatrix):
 
     def __sub__(self, y):
         """Return copy of self.__sub__(y) <==> self-y"""
-        assert isinstance(y, dict)
+        assert isinstance(y, Mapping)
         if self == y:
             f = SpectralMatrix(deepcopy(dict(self)), self.testfunction,
                                self.trialfunction, self.scale-y.scale)
@@ -645,6 +699,7 @@ class Identity(SparseMatrix):
     """
     def __init__(self, shape, scale=1):
         SparseMatrix.__init__(self, {0:1}, shape, scale)
+        self.measure = 1
 
     def solve(self, b, u=None, axis=0):
         if u is None:
@@ -655,6 +710,23 @@ class Identity(SparseMatrix):
         u *= (1/self.scale)
         return u
 
+def BlockMatrices(tpmats):
+    """Return two instances of the :class:`.BlockMatrix` class.
+
+    Parameters
+    ----------
+    tpmats : sequence of :class:`.TPMatrix`'es
+        There should be both boundary matrices from inhomogeneous Dirichlet
+        or Neumann conditions, as well as regular matrices.
+
+    Note
+    ----
+    Use :class:`.BlockMatrix` directly if you do not have any inhomogeneous
+    boundary conditions.
+    """
+    bc_mats = extract_bc_matrices([tpmats])
+    assert len(bc_mats) > 0, 'No boundary matrices - use BlockMatrix'
+    return BlockMatrix(tpmats), BlockMatrix(bc_mats)
 
 class BlockMatrix:
     r"""A class for block matrices
@@ -664,6 +736,14 @@ class BlockMatrix:
         tpmats : sequence of :class:`.TPMatrix` or :class:`.SparseMatrix`
             The individual blocks for the matrix
 
+    Note
+    ----
+    The tensor product matrices must be either boundary
+    matrices or regular matrices, not both. If your problem contains
+    inhomogeneous boundary conditions, then create two BlockMatrices,
+    one for the implicit terms and one for the boundary terms. To this
+    end you can use :class:`.BlockMatrices`.
+
     Example
     -------
     Stokes equations, periodic in x and y-directions
@@ -727,8 +807,9 @@ class BlockMatrix:
     >>> M = BlockMatrix(A00+A01+A10)
 
     """
-    def __init__(self, tpmats):
+    def __init__(self, tpmats, extract_bc_mats=True):
         assert isinstance(tpmats, (list, tuple))
+
         tpmats = [tpmats] if not isinstance(tpmats[0], (list, tuple)) else tpmats
         self.mixedbase = mixedbase = tpmats[0][0].mixedbase
         self.dims = dims = mixedbase.num_components()
@@ -874,7 +955,7 @@ class BlockMatrix:
                     bm[-1].append(d)
         return bmat(bm, format=format)
 
-    def solve(self, b, u=None, constraints=(), return_system=False, Alu=None):
+    def solve(self, b, u=None, constraints=(), return_system=False, Alu=None, BM=None):
         r"""
         Solve matrix system Au = b
 
@@ -934,9 +1015,17 @@ class BlockMatrix:
         else:
             assert u.shape == b.shape
 
+        if BM: # Add contribution to right hand side due to inhomogeneous boundary conditions
+            u.set_boundary_dofs()
+            w0 = np.zeros_like(u)
+            b -= BM.matvec(u, w0)
+
         tpmat = self.get_mats(True)
         axis = tpmat.naxes[0] if isinstance(tpmat, TPMatrix) else 0
-        tp = space.flatten()
+        tp = space.flatten() if hasattr(space, 'flatten') else [space]
+        nvars = b.shape[0] if len(b.shape)>space.dimensions else 1
+        u = u.reshape(1, *u.shape) if nvars == 1 else u
+        b = b.reshape(1, *b.shape) if nvars == 1 else b
         for con in constraints:
             assert len(con) == 3
             assert isinstance(con[0], Integral)
@@ -948,7 +1037,7 @@ class BlockMatrix:
         if space.dimensions == 1:
             s = [0, 0]
             Ai = self.diags((0,))
-            for k in range(b.shape[0]):
+            for k in range(nvars):
                 s[0] = k
                 s[1] = tp[k].slice()
                 gi[self.offset[k][axis]:self.offset[k+1][axis]] = b[tuple(s)]
@@ -956,7 +1045,7 @@ class BlockMatrix:
                 go[:] = Alu.solve(gi)
             else:
                 go[:] = sp.linalg.spsolve(Ai, gi)
-            for k in range(b.shape[0]):
+            for k in range(nvars):
                 s[0] = k
                 s[1] = tp[k].slice()
                 u[tuple(s)] = go[self.offset[k][axis]:self.offset[k+1][axis]]
@@ -968,10 +1057,10 @@ class BlockMatrix:
                 s = [0, 0, 0]
                 if Alu is None:
                     Ai = self.diags(format='csr')
-                gi = np.zeros(space.dim())
-                go = np.zeros(space.dim())
+                gi = np.zeros(space.dim(), dtype=b.dtype)
+                go = np.zeros(space.dim(), dtype=b.dtype)
                 start = 0
-                for k in range(b.shape[0]):
+                for k in range(nvars):
                     s[0] = k
                     s[1] = tp[k].bases[0].slice()
                     s[2] = tp[k].bases[1].slice()
@@ -990,7 +1079,7 @@ class BlockMatrix:
                 else:
                     go[:] = sp.linalg.spsolve(Ai, gi)
                 start = 0
-                for k in range(b.shape[0]):
+                for k in range(nvars):
                     s[0] = k
                     s[1] = tp[k].bases[0].slice()
                     s[2] = tp[k].bases[1].slice()
@@ -1006,14 +1095,14 @@ class BlockMatrix:
                     d0[(axis+1)%2] = i
                     Ai = self.diags(d0)
                     s[ii] = i
-                    for k in range(b.shape[0]):
+                    for k in range(nvars):
                         s[0] = k
                         s[jj] = tp[k].bases[axis].slice()
                         gi[self.offset[k][axis]:self.offset[k+1][axis]] = b[tuple(s)]
                     for con in constraints:
                         Ai, gi = self.apply_constraint(Ai, gi, self.offset[con[0]][axis], i, con)
                     go[:] = sp.linalg.spsolve(Ai, gi)
-                    for k in range(b.shape[0]):
+                    for k in range(nvars):
                         s[0] = k
                         s[jj] = tp[k].bases[axis].slice()
                         u[tuple(s)] = go[self.offset[k][axis]:self.offset[k+1][axis]]
@@ -1027,18 +1116,20 @@ class BlockMatrix:
                     d0[ii-1], d0[jj-1] = i, j
                     Ai = self.diags(d0)
                     s[ii], s[jj] = i, j
-                    for k in range(b.shape[0]):
+                    for k in range(nvars):
                         s[0] = k
                         s[axis+1] = tp[k].bases[axis].slice()
                         gi[self.offset[k][axis]:self.offset[k+1][axis]] = b[tuple(s)]
                     for con in constraints:
                         Ai, gi = self.apply_constraint(Ai, gi, self.offset[con[0]][axis], (i, j), con)
                     go[:] = sp.linalg.spsolve(Ai, gi)
-                    for k in range(b.shape[0]):
+                    for k in range(nvars):
                         s[0] = k
                         s[axis+1] = tp[k].bases[axis].slice()
                         u[tuple(s)] = go[self.offset[k][axis]:self.offset[k+1][axis]]
 
+        u = u.reshape(u.shape[1:]) if nvars == 1 else u
+        b = b.reshape(b.shape[1:]) if nvars == 1 else b
         return u
 
     @staticmethod
@@ -1073,12 +1164,21 @@ class BlockMatrix:
         return A, b
 
 class TPMatrix:
-    """Tensorproduct matrix
+    """Tensor product matrix
 
-    A :class:`.TensorProductSpace` is the outer product of ``D`` bases.
-    A matrix assembled from test and trialfunctions on TensorProductSpaces
-    will, as such, be represented as the outer product of ``D`` smaller matrices,
-    one for each basis. This class represents the complete matrix.
+    A :class:`.TensorProductSpace` is the tensor product of ``D`` univariate
+    function spaces. A normal matrix (a second order tensor) is assembled from
+    bilinear forms (i.e., forms containing both test and trial functions) on
+    one univariate function space. A bilinear form on a tensor product space
+    will assemble to ``D`` outer products of such univariate matrices. That is,
+    for a two-dimensional tensor product you get fourth order tensors (outer
+    product of two matrices), and three-dimensional tensor product spaces leads
+    to a sixth order tensor (outer product of three matrices). This class
+    contains ``D`` second order matrices. The complete matrix is as such the
+    outer product of these ``D`` matrices.
+
+    Note that the outer product of two matrices often is called the Kronecker
+    product.
 
     Parameters
     ----------
@@ -1110,12 +1210,15 @@ class TPMatrix:
         self.naxes = []
         self.global_index = global_index
         self.mixedbase = mixedbase
+        self._issimplified = False
 
     def simplify_diagonal_matrices(self):
         self.naxes = []
         for axis, mat in enumerate(self.mats):
             if not mat:
                 continue
+
+            #if mat.isdiagonal():
             if axis not in self.space.get_nondiagonal_axes():
                 if self.dimensions == 1: # Don't bother with the 1D case
                     continue
@@ -1123,12 +1226,13 @@ class TPMatrix:
                     d = mat[0]    # get diagoal
                     if np.ndim(d):
                         d = self.space[axis].broadcast_to_ndims(d)
-                        d *= mat.scale
+                        d = d*mat.scale
                     self.scale = self.scale*d
                     self.mats[axis] = Identity(mat.shape)
 
             else:
                 self.naxes.append(axis)
+
         # Decomposition
         if len(self.space) > 1:
             s = self.scale.shape
@@ -1143,9 +1247,9 @@ class TPMatrix:
         # this matrix.
         if len(self.naxes) == 1:
             self.pmat = self.mats[self.naxes[0]]
-
         elif len(self.naxes) == 2: # 2 nondiagonal
             self.pmat = self.mats
+        self._issimplified = True
 
     def solve(self, b, u=None):
 
@@ -1364,7 +1468,10 @@ def check_sanity(A, test, trial, measure=1):
     measure : sympy function of coordinate, optional
     """
     N, M = A.shape
-    D = get_dense_matrix(test, trial, measure)[:N, :M]
+    if measure == 1:
+        D = get_dense_matrix(test, trial, measure)[:N, :M]
+    else:
+        D = get_denser_matrix(test, trial, measure)
     Dsp = extract_diagonal_matrix(D)
     for key, val in A.items():
         assert np.allclose(val*A.scale, Dsp[key])
@@ -1392,14 +1499,57 @@ def get_dense_matrix(test, trial, measure=1):
         Additional weight to integral. For example, in cylindrical
         coordinates an additional measure is the radius `r`.
     """
+    K0 = test[0].slice().stop - test[0].slice().start
+    K1 = trial[0].slice().stop - trial[0].slice().start
     N = test[0].N
     x = test[0].mpmath_points_and_weights(N, map_true_domain=False)[0]
     ws = test[0].get_measured_weights(N, measure)
-    v = test[0].evaluate_basis_derivative_all(x=x, k=test[1])
-    u = trial[0].evaluate_basis_derivative_all(x=x, k=trial[1])
+    v = test[0].evaluate_basis_derivative_all(x=x, k=test[1])[:, :K0]
+    u = trial[0].evaluate_basis_derivative_all(x=x, k=trial[1])[:, :K1]
+    A = np.dot(np.conj(v.T)*ws[np.newaxis, :], u)
+    if A.dtype.char in 'FDG':
+        if np.linalg.norm(A.real) / np.linalg.norm(A.imag) > 1e14:
+            A = A.real.copy()
+    return A
+
+def get_denser_matrix(test, trial, measure=1):
+    """Return dense matrix automatically computed from basis
+
+    Use slightly more quadrature points than usual N
+
+    Parameters
+    ----------
+    test : 2-tuple of (basis, int)
+        The basis is an instance of a class for one of the bases in
+
+        - :mod:`.legendre.bases`
+        - :mod:`.chebyshev.bases`
+        - :mod:`.fourier.bases`
+        - :mod:`.laguerre.bases`
+        - :mod:`.hermite.bases`
+        - :mod:`.jacobi.bases`
+
+        The int represents the number of times the test function
+        should be differentiated. Representing matrix row.
+    trial : 2-tuple of (basis, int)
+        As test, but representing matrix column.
+    measure : Sympy expression of coordinate, or number, optional
+        Additional weight to integral. For example, in cylindrical
+        coordinates an additional measure is the radius `r`.
+    """
+    test2 = test[0].get_refined((test[0].N*3)//2)
+    trial2 = trial[0].get_refined((trial[0].N*3)//2)
+
+    K0 = test[0].slice().stop - test[0].slice().start
+    K1 = trial[0].slice().stop - trial[0].slice().start
+    N = test2.N
+    x = test2.mpmath_points_and_weights(N, map_true_domain=False)[0]
+    ws = test2.get_measured_weights(N, measure)
+    v = test[0].evaluate_basis_derivative_all(x=x, k=test[1])[:, :K0]
+    u = trial[0].evaluate_basis_derivative_all(x=x, k=trial[1])[:, :K1]
     return np.dot(np.conj(v.T)*ws[np.newaxis, :], u)
 
-def extract_diagonal_matrix(M, abstol=1e-14, reltol=1e-12):
+def extract_diagonal_matrix(M, abstol=1e-10, reltol=1e-10):
     """Return SparseMatrix version of dense matrix ``M``
 
     Parameters
@@ -1417,7 +1567,7 @@ def extract_diagonal_matrix(M, abstol=1e-14, reltol=1e-12):
     """
     d = {}
     relmax = abs(M).max()
-    dtype = np.float if M.dtype == 'O' else M.dtype # For mpf object
+    dtype = float if M.dtype == 'O' else M.dtype # For mpf object
     for i in range(M.shape[1]):
         u = M.diagonal(i).copy()
         if abs(u).max() > abstol and abs(u).max()/relmax > reltol:
@@ -1430,6 +1580,36 @@ def extract_diagonal_matrix(M, abstol=1e-14, reltol=1e-12):
 
     return SparseMatrix(d, M.shape)
 
+def extract_bc_matrices(mats):
+    """Extract boundary matrices from list of ``mats``
+
+    Parameters
+    ----------
+    mats : list of list of :class:`.TPMatrix`es
+
+    Returns
+    -------
+    list
+        list of boundary matrices.
+
+    Note
+    ----
+    The ``mats`` list is modified in place since boundary matrices are
+    extracted.
+    """
+    bc_mats = []
+    for a in mats:
+        for b in a.copy():
+            if isinstance(b, SparseMatrix):
+                if b.trialfunction[0].boundary_condition() == 'Apply':
+                    bc_mats.append(b)
+                    a.remove(b)
+            elif isinstance(b, TPMatrix):
+                if b.is_bc_matrix():
+                    bc_mats.append(b)
+                    a.remove(b)
+    return bc_mats
+
 def get_dense_matrix_sympy(test, trial, measure=1):
     """Return dense matrix automatically computed from basis
 
@@ -1480,3 +1660,61 @@ def get_dense_matrix_sympy(test, trial, measure=1):
                                       (x, test[0].sympy_reference_domain()[0], test[0].sympy_reference_domain()[1]))
 
     return V
+
+def get_dense_matrix_quadpy(test, trial, measure=1):
+    """Return dense matrix automatically computed from basis
+
+    Using quadpy to compute the integral adaptively with high accuracy.
+    This should be equivalent to integrating analytically with sympy,
+    as long as the integrand is smooth enough and the integral can be
+    found with quadrature.
+
+    Parameters
+    ----------
+    test : 2-tuple of (basis, int)
+        The basis is an instance of a class for one of the bases in
+
+        - :mod:`.legendre.bases`
+        - :mod:`.chebyshev.bases`
+        - :mod:`.fourier.bases`
+        - :mod:`.laguerre.bases`
+        - :mod:`.hermite.bases`
+        - :mod:`.jacobi.bases`
+
+        The int represents the number of times the test function
+        should be differentiated. Representing matrix row.
+    trial : 2-tuple of (basis, int)
+        As test, but representing matrix column.
+    measure : Sympy expression of coordinate, or number, optional
+        Additional weight to integral. For example, in cylindrical
+        coordinates an additional measure is the radius `r`.
+    """
+    import quadpy
+    N = test[0].slice().stop - test[0].slice().start
+    M = trial[0].slice().stop - trial[0].slice().start
+    V = np.zeros((N, M), dtype=test[0].forward.output_array.dtype)
+    x = sp.Symbol('x', real=True)
+
+    if not measure == 1:
+        if isinstance(measure, sp.Expr):
+            s = measure.free_symbols
+            assert len(s) == 1
+            x = s.pop()
+            xm = test[0].map_true_domain(x)
+            measure = measure.subs(x, xm)
+        else:
+            assert isinstance(measure, Number)
+
+    # Weight of weighted space
+    measure *= test[0].weight()
+
+    for i in range(test[0].slice().start, test[0].slice().stop):
+        pi = np.conj(test[0].sympy_basis(i, x=x))
+        for j in range(trial[0].slice().start, trial[0].slice().stop):
+            pj = trial[0].sympy_basis(j, x=x)
+            integrand = sp.simplify(measure*pi.diff(x, test[1])*pj.diff(x, trial[1]))
+            if not integrand == 0:
+                V[i, j] = quadpy.c1.integrate_adaptive(sp.lambdify(x, integrand),
+                                                       test[0].reference_domain())[0]
+
+    return V
