@@ -2,7 +2,7 @@ from .la import TDMA_SymLU, TDMA_SymSolve, PDMA_SymLU, PDMA_SymSolve, \
     TDMA_SymSolve_VC, TDMA_SymLU_VC, PDMA_SymLU_VC, PDMA_SymSolve_VC, \
     LU_Helmholtz, Solve_Helmholtz, LU_Biharmonic, Biharmonic_factor_pr, \
     Biharmonic_Solve, TDMA_O_SymSolve, TDMA_O_SymLU
-from .Matvec import Helmholtz_matvec, Biharmonic_matvec
+from .Matvec import Helmholtz_matvec, Helmholtz_Neumann_matvec, Biharmonic_matvec
 from .outer import outer2D, outer3D
 from .applymask import apply_mask
 from .Cheb import chebval
