@@ -21,9 +21,6 @@ comm = MPI.COMM_WORLD
 __all__ = ('TensorProductSpace', 'VectorSpace', 'TensorSpace',
            'CompositeSpace', 'Convolve')
 
-# Default sympy symbols. Note that order is important
-x, y, z, r, s = psi = sp.symbols('x,y,z,r,s', real=True)
-
 class CurvilinearTransform(Transform):
     """Class for performing forward parallel transform using curvilinear
     coordinates
@@ -63,6 +60,10 @@ class CurvilinearTransform(Transform):
         if dx == 1:
             return
 
+        if isinstance(dx, Number):
+            self.input_array[:] = self.input_array*dx
+            return
+
         sym0 = dx.free_symbols
         m = []
         for sym in sym0:
@@ -170,7 +171,11 @@ class TensorProductSpace(PFFT):
         if not modify_spaces_inplace:
             self.bases = tuple([base.get_unplanned() for base in bases])
 
-        coors = coordinates if coordinates is not None else (psi[:len(bases)],)*2
+        coors = coordinates
+        if coors is None:
+            psi = sp.symbols('x,y,z,r,s', real=True)
+            coors = (psi[:len(bases)],)*2
+
         self.coors = Coordinates(*coors)
         self.hi = self.coors.hi
         self.sg = self.coors.sg
@@ -208,7 +213,7 @@ class TensorProductSpace(PFFT):
             assert sorted(axes[i]) == sorted(set(axes[i]))
 
         if dtype is None:
-            dtype = np.complex if isinstance(self.bases[axes[-1][-1]], C2C) else np.float
+            dtype = complex if isinstance(self.bases[axes[-1][-1]], C2C) else float
 
         dtype = np.dtype(dtype)
         assert dtype.char in 'fdgFDG'
@@ -292,7 +297,8 @@ class TensorProductSpace(PFFT):
 
             self.pencil[1] = pencilA
 
-            FT = functools.partial(CurvilinearTransform, T=self) if not self.hi.prod() == 1 else Transform
+            FT = functools.partial(CurvilinearTransform, T=self) if not coordinates is None else Transform
+            #FT = functools.partial(CurvilinearTransform, T=self) if not self.coors.is_cartesian else Transform
 
             self.forward = FT(
                 [o.forward for o in self.xfftn],
@@ -374,7 +380,7 @@ class TensorProductSpace(PFFT):
 
         self.pencil[1] = pencilA
 
-        FT = functools.partial(CurvilinearTransform, T=self) if not self.hi.prod() == 1 else Transform
+        FT = functools.partial(CurvilinearTransform, T=self) if not self.coors.is_cartesian else Transform
 
         self.backward = Transform(
             [o.backward for o in self.xfftn],
@@ -592,7 +598,7 @@ class TensorProductSpace(PFFT):
                     last_conj_index = M
                 sl = self.local_slice()[axis].start
             x.append(base.map_reference_domain(points[axis]))
-            w.append(base.wavenumbers(bcast=False)[self.local_slice()[axis]].astype(np.float))
+            w.append(base.wavenumbers(bcast=False)[self.local_slice()[axis]].astype(float))
 
         if len(self) == 2:
             output_array = evaluate.evaluate_lm_2D(list(self.bases), output_array, coefficients, x[0], x[1], w[0], w[1], r2c, last_conj_index, sl)
@@ -723,7 +729,7 @@ class TensorProductSpace(PFFT):
         return lm
 
     def local_cartesian_mesh(self, uniform=False):
-        """Return curvilinear mesh
+        """Return Cartesian mesh
 
         Parameters
         ----------
@@ -738,7 +744,7 @@ class TensorProductSpace(PFFT):
         return xx
 
     def cartesian_mesh(self, uniform=False):
-        """Return curvilinear mesh
+        """Return Cartesian mesh
 
         Parameters
         ----------
@@ -768,6 +774,18 @@ class TensorProductSpace(PFFT):
         """The slices of dofs for each dimension"""
         return tuple(base.slice() for base in self.bases)
 
+    @property
+    def use_fixed_gauge(self):
+        """Return True if TensorProductSpace contains only spaces with Neumann
+        boundary conditions.
+        """
+        for base in self.bases:
+            if base.boundary_condition() != 'Neumann':
+                # Any other boundary condition triggers False
+                # It is only Neumann bases that requires a gauge
+                return False
+        return True
+
     def global_shape(self, forward_output=False):
         """Return global shape of arrays for TensorProductSpace
 
@@ -838,6 +856,10 @@ class TensorProductSpace(PFFT):
         if dx == 1:
             return
 
+        if isinstance(dx, Number):
+            u *= dx
+            return u
+
         sym0 = dx.free_symbols
         m = []
         for sym in sym0:
@@ -881,14 +903,16 @@ class TensorProductSpace(PFFT):
         return axes
 
     def get_nondiagonal_axes(self):
-        """Return list of axes that may contain non-diagonal matrices"""
+        """Return list of axes that **may** contain non-diagonal matrices"""
         axes = []
-        if isinstance(self.hi.prod(), sp.Expr):
+        if isinstance(self.coors.hi.prod(), sp.Expr):
             x = 'xyzrs'
-            sym = self.hi.prod().free_symbols
+            sym = self.coors.hi.prod().free_symbols
             msaxes = set()
             for s in sym:
-                msaxes.add(x.index(str(s)))
+                if str(s) in x:
+                    msaxes.add(x.index(str(s)))
+
         for axis, base in enumerate(self):
             if not base.family() == 'fourier':
                 axes.append(axis)
@@ -955,10 +979,21 @@ class TensorProductSpace(PFFT):
             otheraxes = list(range(len(self.bases)))
             otheraxes.remove(axis)
             fj = fun.copy()
+            bcc = None
+            if base.has_nonhomogeneous_bcs:
+                bcc = base.bc.bcs.copy()
             for ax in otheraxes:
                 fj = fj.subs(sd['xyzrs'[ax]], 0.5*(domains[ax][1]-domains[ax][0]))
             if base.N == 0:
-                Tj.append(base.get_adaptive(fun=fj, reltol=reltol, abstol=abstol))
+                par = dict(fun=fj, reltol=reltol, abstol=abstol)
+                if base.has_nonhomogeneous_bcs:
+                    for l, bc in enumerate(bcc):
+                        bc = bc.subs(sd['xyzrs'[ax]], 0.5*(domains[ax][1]-domains[ax][0]))
+                        bcc[l] = bc
+                    par['bc'] = bcc
+                Tj.append(base.get_adaptive(**par))
+                if base.has_nonhomogeneous_bcs:
+                    Tj[-1].bc = BoundaryValues(Tj[-1], bc=base.bc)
             else:
                 Tj.append(base)
         return TensorProductSpace(self.subcomm, Tj, axes=self.axes,
@@ -1228,11 +1263,11 @@ class CompositeSpace:
 
 
 class VectorSpace(CompositeSpace):
-    """Vector of :class:`.TensorProductSpace`s
+    """Vector of :class:`.TensorProductSpace` instances.
 
     This is simply a special :class:`.CompositeSpace` where the number of
-    :class:`.TensorProductSpace`s equals the geometrical dimension of the
-    problem.
+    instances of the :class:`.TensorProductSpace` class equals the geometrical
+    dimension of the problem.
 
     For example, a TensorProductSpace created by a tensorproduct of 2 1D
     spaces, will have vectors of length 2. A TensorProductSpace created from 3
@@ -1240,8 +1275,8 @@ class VectorSpace(CompositeSpace):
 
     Parameters
     ----------
-    space : :class:`.TensorProductSpace` or list of ndim :class:`.TensorProductSpace`s
-        Spaces to create vector from
+    space : :class:`.TensorProductSpace` or list of ndim instances of the
+    :class:`.TensorProductSpace` class to create vector from.
 
     """
 
@@ -1295,7 +1330,7 @@ class VectorSpace(CompositeSpace):
 class TensorSpace(VectorSpace):
     """A special :class:`.CompositeSpace` for second rank tensors.
 
-    For example, a TensorProductSpace created by a tensorproduct of 2 1D
+    A TensorProductSpace created by a tensorproduct of 2 1D
     bases, will have tensors of length 4. A TensorProductSpace created from 3
     1D bases will have tensors of length 9.
 
@@ -1438,6 +1473,8 @@ class BoundaryValues:
             for i, bci in enumerate(bc):
                 if isinstance(bci, (Number, sp.Expr, np.ndarray)):
                     self.bcs[i] = bci
+                elif bci is None:
+                    self.bcs[i] = None
                 else:
                     raise NotImplementedError
 
@@ -1463,6 +1500,9 @@ class BoundaryValues:
         To modify boundary conditions on the fly, modify first self.bc and then
         call this function. The boundary condition can then be applied as before.
         """
+        from shenfun.utilities import split
+        from shenfun import project
+
         self.axis = this_base.axis
         self.base = this_base
         self.tensorproductspace = T
@@ -1470,13 +1510,12 @@ class BoundaryValues:
         if isinstance(T, SpectralBase):
             pass
         elif this_base.has_nonhomogeneous_bcs:
-            # Setting the Dirichlet boundary condition in a TensorProductSpace
+            # Setting an inhomogeneous boundary condition in a TensorProductSpace
             # is more involved than for a single dimension, and the routine will
-            # depend on the order of the bases. If the Dirichlet space is the last
-            # one, then the boundary condition is applied directly. If there is
-            # one other space to the right, then one transform needs to
-            # be performed on the bc data first. For two other spaces to the right,
-            # two transforms need to be executed.
+            # depend on the order of the bases. Basically, the inhomogeneous value
+            # needs to be projected to a space consisting of all the other function
+            # spaces, except the inhomogeneous one. This would be very easy if it was
+            # not for MPI.
             axis = None
             number_of_bases_after_this = 0
             for axes in reversed(T.axes):
@@ -1495,11 +1534,12 @@ class BoundaryValues:
                 return
 
             # Set boundary values
-            # These are values set at the end of a transform in Dirichlet space,
-            # but before any other transforms
-            # Shape is like real space, since Dirichlet does not alter shape
+            # These are values set at the end of a transform in an
+            # inhomogeneous space, but before any other transforms
+
             b = Array(T)
             s = T.local_slice(False)[self.axis]
+            num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) == None)
 
             for j, bci in enumerate(self.bc):
                 if isinstance(bci, sp.Expr):
@@ -1516,67 +1556,57 @@ class BoundaryValues:
                         Yi.append(X[k][this_base.si[j]])
                     f_bci = lbci(*Yi)
 
-                    # Put the Dirichlet value in the position of the bc dofs
+                    # Put the value in the position of the bc dofs
                     if s.stop == int(this_base.N*this_base.padding_factor):
-                        b[this_base.si[-(len(self.bc))+j]] = f_bci
+                        if num_bcs == 1: # e.g., UpperDirichlet
+                            b[this_base.si[-1]] = f_bci
+                        else:
+                            b[this_base.si[-(len(self.bc))+j]] = f_bci
 
                 elif isinstance(bci, (Number, np.ndarray)):
                     if s.stop == int(this_base.N*this_base.padding_factor):
-                        b[this_base.si[-(len(self.bc))+j]] = bci
+                        if num_bcs == 1:
+                            b[this_base.si[-1]] = bci
+                        else:
+                            b[this_base.si[-(len(self.bc))+j]] = bci
+
+                elif bci is None:
+                    continue
 
                 else:
                     raise NotImplementedError
 
-            if len(T.get_nonhomogeneous_axes()) == 1:
-                if number_of_bases_after_this == 0:
-                    # Dirichlet base is the first to be transformed
-                    b_hat = b
+                if len(T.get_nonhomogeneous_axes()) == 1:
+                    from .spectralbase import FuncWrap
 
-                elif number_of_bases_after_this == 1:
-                    T.forward._xfftn[0].input_array[...] = b
+                    if number_of_bases_after_this == 0:
+                        # Inhomogeneous base is the first to be transformed
+                        b_hat = b
 
-                    T.forward._xfftn[0]()
-                    arrayA = T.forward._xfftn[0].output_array
-                    arrayB = T.forward._xfftn[1].input_array
-                    T.forward._transfer[0](arrayA, arrayB)
-                    b_hat = arrayB.copy()
+                    else:
+                        i = number_of_bases_after_this
+                        u_hat = T.forward._xfftn[i].input_array
+                        fun = FuncWrap(lambda x, y: x, u_hat, u_hat) # Do-nothing
+                        fwd = Transform(list(T.forward._xfftn[:i])+[fun],
+                                 T.forward._transfer[:i],
+                                 [[], []])
+                        b_hat = fwd(b).copy()
 
-                elif number_of_bases_after_this == 2:
-                    T.forward._xfftn[0].input_array[...] = b
+                    # Now b_hat contains the correct slices in slm1 and slm2
+                    # These are the values to use on intermediate steps.
+                    # If for example a Dirichlet space is squeezed between two Fourier spaces
+                    if num_bcs == 1:
+                        self.bcs[j] = b_hat[this_base.si[-1]].copy()
+                    else:
+                        self.bcs[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
+                    b_hat = T.forward(b).copy()
+                    if num_bcs == 1:
+                        self.bcs_final[j] = b_hat[this_base.si[-1]].copy()
+                    else:
+                        self.bcs_final[j] = b_hat[this_base.si[-(len(self.bc))+j]].copy()
 
-                    T.forward._xfftn[0]()
-                    arrayA = T.forward._xfftn[0].output_array
-                    arrayB = T.forward._xfftn[1].input_array
-                    T.forward._transfer[0](arrayA, arrayB)
-
-                    T.forward._xfftn[1]()
-                    arrayA = T.forward._xfftn[1].output_array
-                    arrayB = T.forward._xfftn[2].input_array
-                    T.forward._transfer[1](arrayA, arrayB)
-                    b_hat = arrayB.copy()
-
-                # Now b_hat contains the correct slices in slm1 and slm2
-                # These are the values to use on intermediate steps. If for example the Dirichlet space is squeezed between two Fourier spaces
-                for i in range(len(self.bc)):
-                    self.bcs[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
-
-                # Final (the values to set on fully transformed functions)
-                T.forward._xfftn[0].input_array[...] = b
-                for i in range(len(T.forward._transfer)):
-
-                    T.forward._xfftn[i]()
-                    arrayA = T.forward._xfftn[i].output_array
-                    arrayB = T.forward._xfftn[i+1].input_array
-                    T.forward._transfer[i](arrayA, arrayB)
-
-                T.forward._xfftn[-1]()
-                b_hat = T.forward._xfftn[-1].output_array
-                for i in range(len(self.bc)):
-                    self.bcs_final[i] = b_hat[this_base.si[-(len(self.bc))+i]].copy()
-
-            else: # 2 non-homogeneous directions
+            if len(T.get_nonhomogeneous_axes()) == 2:
                 assert len(T.bases) == 2, 'Only implemented for 2D'
-                from shenfun import project
                 bases = []
                 for axis, base in enumerate(T.bases):
                     if not base is this_base:
@@ -1584,23 +1614,36 @@ class BoundaryValues:
 
                 other_base = bases[0]
                 ua = Array(other_base)
-                for j, bcj in enumerate(other_base.bc.bc.copy()):
-                    xj = this_base.domain[j]
-                    sym = sp.sympify(bcj).free_symbols
-                    if len(sym) == 1:
-                        s = bcj.subs(sym.pop(), xj)
-                        other_base.bc.bc[j] = s
-                        other_base.bc.bcs[j] = s
-                        other_base.bc.bcs_final[j] = s
-                    else:
-                        other_base.bc.bc[j] = bcj
-                        other_base.bc.bcs[j] = bcj
-                        other_base.bc.bcs_final[j] = bcj
-
-                for j in range(len(self.bc)):
-                    ua[:] = b[this_base.si[-(len(self.bc))+j]]
-                    self.bcs_final[j] = project(ua, other_base)
+                bc_this = this_base.bc.bc.copy()
+                df = 2./(other_base.domain[1]-other_base.domain[0])
+                for i in range(2): # x = -1 and then x = 1
+                    bcj = bc_this[i]
+                    for j in range(2): # y = -1 and then y = 1
+                        xj = other_base.domain[j]
+                        sym = sp.sympify(bcj).free_symbols
+                        if len(sym) == 1:
+                            xx = sym.pop()
+                            if j == 0 and other_base.boundary_condition() == 'NeumannDirichlet':
+                                s = bcj.diff(xx, 1).subs(xx, xj)/df
+                            elif j == 1 and other_base.boundary_condition() == 'DirichletNeumann':
+                                s = bcj.diff(xx, 1).subs(xx, xj)/df
+                            elif j == 0 and other_base.boundary_condition() == 'UpperDirichletNeumann':
+                                xj = other_base.domain[1]
+                                s = bcj.subs(xx, xj)
+                            elif j == 1 and other_base.boundary_condition() == 'UpperDirichletNeumann':
+                                s = bcj.diff(xx, 1).subs(xx, xj)/df
+                            else:
+                                s = bcj.subs(xx, xj)
 
+                            other_base.bc.bc[j] = s
+                            other_base.bc.bcs[j] = s
+                            other_base.bc.bcs_final[j] = s
+                        else:
+                            other_base.bc.bc[j] = bcj
+                            other_base.bc.bcs[j] = bcj
+                            other_base.bc.bcs_final[j] = bcj
+                    ua[:] = b[this_base.si[-(len(self.bc))+i]]
+                    self.bcs_final[i] = project(ua, other_base)
 
     def add_to_orthogonal(self, u, uh):
         """Add contribution from boundary functions to `u`
@@ -1655,12 +1698,23 @@ class BoundaryValues:
 
         """
         M = len(self.bc)
+        num_bcs = len(self.bc) - np.count_nonzero(np.array(self.bc) == None)
+
         if final is True:
             for i in range(M):
-                u[self.base.si[-(M)+i]] = self.bcs_final[i]
+                if num_bcs == 1:
+                    if self.bcs_final[i] is None:
+                        continue
+                    u[self.base.si[-1]] = self.bcs_final[i]
+                else:
+                    u[self.base.si[-(M)+i]] = self.bcs_final[i]
 
         else:
             for i in range(M):
+                if num_bcs == 1:
+                    if self.bcs[i] is None:
+                        continue
+                    u[self.base.si[-1]] = self.bcs[i]
                 u[self.base.si[-(M)+i]] = self.bcs[i]
 
     def has_nonhomogeneous_bcs(self):
@@ -1691,7 +1745,7 @@ def some_basic_tests():
         f_g_hat = pyfftw.interfaces.numpy_fft.rfftn(f_g, axes=(0, 1, 2, 3))
     else:
         f_g = np.zeros(T.shape())
-        f_g_hat = np.zeros(T.shape(), dtype=np.complex)
+        f_g_hat = np.zeros(T.shape(), dtype=complex)
 
     # Distribute test data to all ranks
     comm.Bcast(f_g, root=0)
